#include "CircleNode.h"

#include "SceneModel.h"
#include "AbstractMesh.h"
#include "MeshPrimitives.h"
#include "SceneInfo.h"
#include "Circle.h"

CircleNode::CircleNode(const Point& location, double radius)
:    GeometryNode(location),
     mRadius(radius)
{
    mMinorRadius = mRadius;
}

CircleNode::Type CircleNode::getType()
{
    return Type::Circle;
}

double CircleNode::getRadius() const
{
    return mRadius;
}

void CircleNode::setRadius(double radius)
{
    if (mRadius != radius)
    {
        mRadius = radius;
        mTransformIsDirty = true;
    }
}

void CircleNode::setMinorRadius(double radius)
{
    if (mMinorRadius != radius)
    {
        mMinorRadius = radius;
        mTransformIsDirty = true;
    }
}

void CircleNode::createOrUpdateGeometry(SceneInfo* sceneInfo)
{
    if (sceneInfo->mSupportsGeometryPrimitives)
    {
        double unit_circle_radius = 0.5;
        auto circle = std::make_unique<Circle>(Point{ 0, 0 }, unit_circle_radius);
        circle->setMinorRadius(unit_circle_radius*mMinorRadius/mRadius);
        mBackgroundItem = std::make_unique<SceneModel>(std::move(circle));
    }
    else
    {
        auto mesh = MeshPrimitives::buildCircleAsTriMesh();
        mBackgroundItem = std::make_unique<SceneModel>(std::move(mesh));
    }
    mBackgroundItem->setName(mName + "_Model");
}

void CircleNode::updateTransform()
{
    mBackgroundItem->updateTransform({ mLocation, 2 * mRadius, 2 * mRadius });
}
